# vim:set ft=sh:
#
# Common library function for Open-Xchange test
#
CURL_BIN="/usr/bin/curl"
[ -d "${OX_TMP:-}" ] || OX_TMP=$(mktemp -d) && trap "rm -rf '$OX_TMP'" EXIT
: ${OX_SESSION_FILE:="${OX_TMP}/session.txt"}
: ${OX_COOKIES:="${OX_TMP}/cookies"}
OX_HTTP_HOST="127.0.0.1"

: ${TESTLIBPATH:=/usr/share/ucs-test/lib}
. "$TESTLIBPATH/base.sh" || exit 137
. "$TESTLIBPATH/user.sh" || exit 137
. "$TESTLIBPATH/group.sh" || exit 137
. "$TESTLIBPATH/random.sh" || exit 137

ox_ajax () {
	# <http://oxpedia.org/wiki/index.php?title=HTTP_API>
	local url="$1"
	shift
	local IFS='&'
	"$CURL_BIN" \
		--silent --show-error \
		--location --location-trusted \
		--insecure \
		--cookie "$OX_COOKIES" --cookie-jar "$OX_COOKIES" \
		"http://${OX_HTTP_HOST}/ajax/${url}?${*}"
}
ox_session () {
	sed -rne 's/^.*"session":"([0-9a-f]+)".*$/\1/p' "$OX_SESSION_FILE"
}

create_ox_user () {
	local username="$1"
	local mail="$2"
	local firstname="${3:-$(random_string)}"
	local lastname="${4:-$(random_string)}"
	shift 4 || shift 2 || return 1

	udm-test users/user create \
		--position "cn=users,$ldap_base" \
		--set username="$username" \
		--set firstname="$firstname" \
		--set lastname="$lastname" \
		--set password="univention" \
		--set mailPrimaryAddress="$mail" \
		"$@" ||
		fail_test 110 "Failed to create ox-user"
}

create_non_ox_user () {
	local username="$1"
	shift 1

	udm-test users/user create \
		--position "cn=users,$ldap_base" \
		--set username="$username" \
		--set firstname="$(random_string)" \
		--set lastname="$(random_string)" \
		--set password="univention" \
		"$@" ||
		fail_test 110 "Failed to create non-ox-user"
}

change_password () {
	local username="$1"
	local new_password="$2"
	shift 2

	udm-test users/user modify \
		--dn "uid=$username,cn=users,$ldap_base" \
		--set password="$new_password" \
		"$@" ||
		fail_test 110 "Failed to change password"
}

ox_login () {
	local username="$1"
	local password="$2"
	shift 2
	local i error_message
	declare -a cmd=(ox_ajax "login" action=login "name=${username}" "password=${password}")

	for ((i=0;i<6;i++)); do
		echo "${cmd[*]}"
		if ! "${cmd[@]}" >"$OX_SESSION_FILE"
		then
			echo "Connection failed"
			return 1
		fi

		echo "SESSION_FILE: $(cat "$OX_SESSION_FILE")"

		error_message="$(grep '"error"' "$OX_SESSION_FILE")"

		if egrep -1 '("SERVICE_DOWN"|Service Temporarily Unavailable)' "$OX_SESSION_FILE"
		then
			echo "Sleeping for 10 seconds and try again"
			sleep 10
			continue
		fi

		break
	done

	if [ -n "$error_message" ]; then
		echo "Login failed"
		return 1
	else
		echo "Login passed"
		return 0
	fi
}

ox_group_randomname () {
	# univention-ox-common/modules/univention/admin/syntax.d/50_ox.py # gid_ox
	local prefix="${_lowerletters}${_upperletters}"
	local main="${_lowerletters}${_upperletters}${_ciphers}._ -"
	local suffix="${_lowerletters}${_upperletters}${_ciphers}"
	random_chars --prefix "$prefix" --suffix "$suffix" 20 "$main"
}

ox_csv_setup () {
	set -e -o errtrace
	csv="$OX_TMP/csv"
	trap "[ -s '$csv' ] && cat '$csv'" ERR
	export PATH="$PATH:/opt/open-xchange/sbin"
	eval "$(ucr shell ox/context/id)"
	CONTEXT="${ox_context_id:=10}"
	if [ "$CONTEXT" = "10" ]; then
		CONTEXTADMIN="oxadmin"
	else
		CONTEXTADMIN="oxadmin-context${CONTEXT}"
	fi
	DOMAIN="$(ucr get domainname)"
	CSVANY='("([^"]|"")*")?'
	CSVBOOL='(false|true)'
}
ox_check_csv () {
	python -c '
from sys import argv, exit
from re import match
from univention.ox.listener_tools import CSV_DELIMITER, CSV_QUOTECHAR, DictReader

result = 0

csv_file = open(argv[1], "r+")
reader = DictReader(csv_file, delimiter=CSV_DELIMITER, quotechar=CSV_QUOTECHAR)
for row in reader:
	for arg in argv[2:]:
		key, expected = arg.split("=", 1)
		actual = row[key]
		if expected.startswith(chr(39)) and expected.endswith(chr(39)):
			expected = expected[1:-1]
		if not match(expected, actual):
			print "%s: expected %r, got %r" % (key, expected, actual)
			result = 1

expected = set(arg.split("=", 1)[0] for arg in argv[2:])
actual = set(reader.fieldnames)
if expected != actual:
	print "expected %r, got %r" % (expected, actual)
	result = 1

exit(result)
' "$csv" "$@"
}

ox_sudo () {
	local cmd="$1"
	shift
	case "${cmd##*/}" in
	*context|*database|*filestore)
		"$cmd" \
			--adminuser oxadminmaster \
			--adminpass "$(cat '/etc/ox-secrets/master.secret')" \
			"$@"
		;;
	*group|*resource|*user)
		"$cmd" \
			--adminuser "$CONTEXTADMIN" \
			--adminpass "$(cat "/etc/ox-secrets/context${CONTEXT}.secret")" \
			--contextid "$CONTEXT" \
			"$@"
		;;
	*)
		"$cmd" "$@"
		;;
	esac
}
